#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#define  LED_ON    TRUE
#define  LED_OFF   FALSE
#define  DIO_LOW   0
#define  DIO_HIGH  8

static gboolean  bLED_stat[16];

static char *sOn  = "On";
static char *sOff = "Off";
static char  dio_cmd[32];

int dio_init(void);
int dio_exit(void);
int dio_write(char *cmd);

static void
SetLED_stat(int Start, 
            int End,
            gboolean  BitOn, /* TRUE: on  FALSE: off */
            GtkButton  *button )
{
  GtkToggleButton *togglebutton;
  GtkWidget *widget;
  char widget_name[8];

  int  LED_no;

  for (LED_no = Start; LED_no <= End; LED_no++) {
    bLED_stat[LED_no] = BitOn;
    sprintf(widget_name, "bLED%d", LED_no+1);
    widget = lookup_widget( GTK_WIDGET(button), (const gchar *)widget_name );
    togglebutton = GTK_TOGGLE_BUTTON(widget);
    gtk_toggle_button_set_active( togglebutton, BitOn );
  }
}


static int
Out_LED_BitStat( int BitNo,
              GtkToggleButton *togglebutton )
{
  char  *StrOnOff;

  /* TRUE: on  FALSE: off */
  bLED_stat[BitNo] = gtk_toggle_button_get_active( togglebutton );
  StrOnOff = ( bLED_stat[BitNo] == TRUE) ? "On" : "Off";
  g_print("LED%d is %s\n", BitNo + 1, StrOnOff);

  sprintf( dio_cmd, "SetBit%s:%d\n", StrOnOff, BitNo);
  dio_write(dio_cmd);

  return 0;
}

static int
Out_LED_ByteStat( int  sBit,
                  gboolean  BitOn, /* TRUE: on  FALSE: off */
                  GtkButton  *button )
{
  char  *StrOnOff;
  char  *StrLowHigh;
  int   ByteVal;

  SetLED_stat( sBit, sBit + 7, BitOn, button );

  if (BitOn == TRUE) {
    StrOnOff = "On";
    ByteVal = 0xff;
  } else {
    StrOnOff = "Off";
    ByteVal = 0x00;
  }
  StrLowHigh = ( sBit == 0 ) ? "Low" : "High";
  g_print("%s LED%d-%d is %s\n", 
              StrLowHigh, sBit + 1, sBit + 8, StrOnOff);

  sprintf( dio_cmd, "Write%sByte:0x%02X\n", StrLowHigh, ByteVal);
  dio_write(dio_cmd);

  return 0;
}

static int
Out_LED_WordStat( gboolean BitOn,  /* TRUE: on  FALSE: off */
                  GtkButton  *button )
{
  int   WordVal;


  SetLED_stat( 0, 15, BitOn, button ); 
  WordVal = (BitOn == TRUE) ? 0xffff : 0x0000 ;
  g_print("All LED are %s\n",
                (( BitOn == TRUE ) ? sOn : sOff ) );

  sprintf( dio_cmd, "WriteWord:0x%04X\n", WordVal);
  dio_write(dio_cmd);

  return 0;
}


/* ========================================================== */

gboolean
on_Topwin_delete_event                 (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  int  result;

  result = dio_exit();
  g_print("dio_exit: %d\n", result);

  gtk_main_quit();

  g_print("on_Topwin_delete_event: quit\n");

  return FALSE;
}


void
on_bLED1_toggled                       (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  Out_LED_BitStat( 0, togglebutton );
}


void
on_bLED2_toggled                       (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  Out_LED_BitStat( 1, togglebutton );
}


void
on_bLED3_toggled                       (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  Out_LED_BitStat( 2, togglebutton );
}


void
on_bLED4_toggled                       (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  Out_LED_BitStat( 3, togglebutton );
}


void
on_bLED5_toggled                       (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  Out_LED_BitStat( 4, togglebutton );
}


void
on_bLED6_toggled                       (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  Out_LED_BitStat( 5, togglebutton );
}


void
on_bLED7_toggled                       (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  Out_LED_BitStat( 6, togglebutton );
}


void
on_bLED8_toggled                       (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  Out_LED_BitStat( 7, togglebutton );
}


void
on_bLowOn_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
  Out_LED_ByteStat( DIO_LOW, LED_ON, button );
}


void
on_bLowOff_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  Out_LED_ByteStat( DIO_LOW, LED_OFF, button );
}


void
on_bLED9_toggled                       (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  Out_LED_BitStat( 8, togglebutton );
}


void
on_bLED10_toggled                      (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  Out_LED_BitStat( 9, togglebutton );
}


void
on_bLED11_toggled                      (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  Out_LED_BitStat( 10, togglebutton );
}


void
on_bLED12_toggled                      (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  Out_LED_BitStat( 11, togglebutton );
}


void
on_bLED13_toggled                      (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  Out_LED_BitStat( 12, togglebutton );
}


void
on_bLED14_toggled                      (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  Out_LED_BitStat( 13, togglebutton );
}


void
on_bLED15_toggled                      (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  Out_LED_BitStat( 14, togglebutton );
}


void
on_bLED16_toggled                      (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  Out_LED_BitStat( 15, togglebutton );
}


void
on_bHighOn_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  Out_LED_ByteStat( DIO_HIGH, LED_ON, button );
}


void
on_bHighOff_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  Out_LED_ByteStat( DIO_HIGH, LED_OFF, button );
}


void
on_bWordOn_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  Out_LED_WordStat( LED_ON, button );
}


void
on_bWordOff_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  Out_LED_WordStat( LED_OFF, button );
}


void
on_combo_entry1_changed                (GtkEditable     *editable,
                                        gpointer         user_data)
{
  char *str;

  str = gtk_entry_get_text(GTK_ENTRY(editable));
  DevNameChanged( str );
}

